import json, requests


class YandexGPTEntryPoint:
    def __init__(self, token, folder_id):
        iam_token = requests.post(
            url='https://iam.api.cloud.yandex.net/iam/v1/tokens', 
            data=json.dumps({'yandexPassportOauthToken': token})
            ).json()['iamToken']
        self.folder_id = folder_id
        self.text_url = 'https://llm.api.cloud.yandex.net/foundationModels/v1/completion'
        self.emb_url = 'https://llm.api.cloud.yandex.net:443/foundationModels/v1/textEmbedding'
        self.doc_uri = f'emb://{self.folder_id}/text-search-doc/latest'
        self.query_uri = f'emb://{self.folder_id}/text-search-query/latest'
        self.headers = {
            'Authorization': f'Bearer {iam_token}', 
            'Content-Type': 'application/json'
            }
        self.DIM = 256
        self.ZEROS = [0 for _ in range(self.DIM)]
        self.ERROR_MESSAGE = ''
        self.warmup()

    def get_response(self, total_input: str) -> str:
        data = {
            'model_uri': f'gpt://{self.folder_id}/yandexgpt/latest',
            'messages': [
                {
                    'role': 'user',
                    'text': total_input,
                }
            ],
        }
        try:
            response = requests.post(
                url=self.text_url, 
                headers=self.headers, 
                json=data
                ).json()
            return response['result']['alternatives'][0]['message']['text']
        except:
            return self.ERROR_MESSAGE

    def get_embeddings(self, total_input, input_is_long=False) -> list:
        data = {
            'modelUri': self.doc_uri if input_is_long else self.query_uri, 
            'text': total_input
            }
        try:
            response = requests.post(
                url=self.emb_url, 
                headers=self.headers, 
                json=data
                ).json()
            return response['embedding']
        except:
            return self.ZEROS

    def warmup(self) -> None:
        assert self.get_response('Прогрев') != self.ERROR_MESSAGE, 'Нет доступа к ллм!'
        assert self.get_embeddings('Прогрев') != self.ZEROS, 'Нет доступа к ллм!'
        assert self.get_embeddings('Прогрев', input_is_long=True) != self.ZEROS, 'Нет доступа к ллм!'
                


import yaml, os, codecs
from .singleton import singleton


@singleton
class ConfigSingleton:
    def __init__(self, common_config_dir=None, branch_config_dir=None):
        common = self.load_config(common_config_dir)
        branch = self.load_config(branch_config_dir)
        self.config = {**common, **branch}
        assert len(self.config), 'Error: empty config!'

    def get(self):
        return self.config

    @staticmethod
    def load_config(path=None):
        if path is not None:
            if path.endswith('/'): 
                path = path[:-1]
            assert os.path.exists(path), 'Error: config unavailable!'
            with codecs.open(f'{path}/config.yaml', encoding='utf-8') as file:
                return yaml.load(file, yaml.Loader)
        return {}



import logging
from pythonjsonlogger import jsonlogger
from sys import stdout

from ..config.singleton import singleton
from ..dig_ass.db import make_session_id


class JsonFormatter(jsonlogger.JsonFormatter):
    pass
    ####
    # additional json formatting possible
    ####

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.json_ensure_ascii = False


@singleton
class LoggerSingleton:
    def __init__(self, loggerConfig) -> None:
        self.logger = logging.getLogger(name=loggerConfig['name'])
        self.logger.setLevel(loggerConfig['level_common'])

        stdoutHandler = logging.StreamHandler(stdout)

        # configure logging to stdout
        # stdout_json == True -> set json Formatter
        stdoutJsonFlag = False
        try:
            stdoutJsonFlag = loggerConfig['stdout_json']  # optional config field
        except:
            pass

        if stdoutJsonFlag:
            stdoutFormatter = JsonFormatter(loggerConfig['format_stdout'])
        else:
            stdoutFormatter = logging.Formatter(loggerConfig['format_stdout'])

        stdoutHandler.setFormatter(stdoutFormatter)
        stdoutHandler.setLevel(loggerConfig['level_stdout_handler'])
        self.logger.addHandler(stdoutHandler)

        # configure logging to file
        writeToFileFlag = False
        try:
            writeToFileFlag = loggerConfig['write_to_file']  # optional config field
        except:
            pass

        if writeToFileFlag:
            logFilePath = f'{loggerConfig["file_dir"]}/{loggerConfig["name"]}_{make_session_id()}.json'
            fileHandler = logging.FileHandler(logFilePath, mode='a')
            jsonFormatter = JsonFormatter(loggerConfig['format_file'])
            fileHandler.setFormatter(jsonFormatter)
            fileHandler.setLevel(loggerConfig['level_file_handler'])
            self.logger.addHandler(fileHandler)

    def get(self):
        return self.logger
